@@ -2,19 +2,14 @@ package Net::Domain::Regex;
 
 use strict;
 
-use version; our $VERSION = qv('0.2.1');
+our $VERSION = 0.0001_01;
 
 our $LOCAL = '/tmp/effective_tld_names.dat';
 our $CACHE = '/tmp/effective_tld_names.dat.cache';
 our $SOURCE = 'http://mxr.mozilla.org/mozilla-central/source/netwerk/dns/effective_tld_names.dat?raw=1';
 
 use LWP::UserAgent;
-
-sub import {
-	if( grep { /:pdata/ } @_ ){
-		$SOURCE = 'https://raw.github.com/petermblair/Perl-CPAN/master/Net-Domain-Regex/misc/tld.txt';
-	}
-}
+use Cache::FileCache;
 
 sub new {
 	my $class = shift;
@@ -28,11 +23,20 @@ sub new {
 
 	my $o = bless $args => $class;
 
-	unless( -e $o->{local} ){
+	unless( -e $self->{local} ){
 		$o->pull;
 	}
 
-	$o->refresh;
+	$self->{filecache} = Cache::FileCache->new;
+
+	unless( $self->{filecache}->get( 'sld' ) && $self->{filecache}->get( 'tld' ) ){
+		$o->refresh;
+		$self->{filecache}->set( 'sld', $o->{sld}, "1 month" );
+		$self->{filecache}->set( 'tld', $o->{tld}, "1 month" );
+	}
+
+	$o->{sld} = $self->{filecache}->get( 'sld' );
+	$o->{tld} = $self->{filecache}->get( 'tld' );
 
 	return $o;
 }
@@ -40,30 +44,32 @@ sub new {
 sub refresh {
 	my $self = shift;
 
-	use open qw(:std :utf8);
 	open FD, "<$self->{local}";
 
-	my $tlds = {};
-	my $slds = {};
+	my $tld = {};
+	my $sld = {};
 
 	while( <FD> ){
 		chomp;
 
 		if(/^(\S[^\.\s]+)$/){
-			$tlds->{$1}++;
+			$tld->{$1}++;
 		}
-		elsif ( /^\S[^\.\s]+\.(.+)$/ && exists $tlds->{$1} ) {
-			$slds->{$_}++;
+
+		for my $t ( keys %$tld ){
+			if( /^(\S[^\.\s]+\.$t)$/ ){
+				$sld->{$1}++;
+			}
 		}
 	}
 
 	# any manual overrides
 	for( qw/ co.uk / ){
-		$tlds->{"$_"}++;
+		$tld->{"$_"}++;
 	}
 
-	$self->{tld} = $tlds;
-	$self->{sld} = $slds;
+	$self->{tld} = $tld;
+	$self->{sld} = $sld;
 }
 
 sub pull {
@@ -85,23 +91,13 @@ sub pull {
 sub generate_regex {
 	my $self = shift;
 
-	my @a;
-
-	for( keys %{$self->{sld}} ){
-		push( @a, $_ );
-	}
-
-	for( keys %{$self->{tld}} ){
-		push( @a, $_ );
-	}
-
-	my @atld = sort { length $b cmp length $a } @a;
-
-	my $tld = join( "|", @atld );
+	my $sld = join( "|", sort keys %{$self->{sld}} );
+	$sld =~ s/\./\\./g;
+	my $tld = join( "|", sort keys %{$self->{tld}} );
 	$tld =~ s/\./\\./g;
 
 	#my $regex = "((?:[a-zA-Z0-9]\\w+\\.)(($tld)|($sld)))";
-	my $regex = "((?:[a-zA-Z0-9][\\w\\-]+\\.)+(com|net|org|edu|$tld))\$";
+	my $regex = "((?:[a-zA-Z0-9]\\w+\\.)+(com|net|org|edu|$tld))\$";
 
 	return $regex;
 }
@@ -115,7 +111,7 @@ sub match {
 	
 	#print "Regex: [$regex]\n";
 
-	my @tokens = split /[^\w\.\-]/, $target;
+	my @tokens = split /[^\w\.]/, $target;
 	my @results;
 
 	for my $target( @tokens ){
